#!/usr/bin/env python

# Locking-Beispiel
# basiert auf
# https://www.bogotobogo.com/python/Multithread/python_multithreading_Synchronization_Lock_Objects_Acquire_Release.php

# Anpassungen: hge, 2023-12-15

from threading import Thread as create_thread, \
    Lock as create_mutex
from time import sleep
import logging
import random

logging.basicConfig(level=logging.DEBUG,
                    format='(%(threadName)-9s) %(message)s',)
                    
class Counter(object):
    def __init__(self, start = 0):
        self.lock = create_mutex()      # Lock = Mutex
        self.value = start
    def bad_increment(self):
        tmp = self.value
        sleep(0.01)
        tmp = tmp + 1
        self.value = tmp
    def good_increment(self):
        self.lock.acquire()     # Mutex sperren
        try:
            tmp = self.value
            sleep(0.01)         # Chance fuer Thread-Wechsel
            tmp = tmp + 1
            self.value = tmp
        finally:
            self.lock.release() # Mutex freigeben
    increment = bad_increment   # <-- hier anpassen

def worker(c):
    for i in range(10):
        r = random.random()/50
        logging.debug('Sleeping %0.02f ms', r*1000)
        sleep(r)
        c.increment()

counter = Counter()

# Threads erzeugen
threads = []
for i in range(2):
    # Thread erzeugen, der worker(counter) ausfuehrt
    t = create_thread(target=worker, args=(counter,))
    t.start()
    threads.append(t)

# Fertige Threads einsammeln
for t in threads: t.join()

print('Counter: %d' % counter.value)

